#include "Compiler.h"
#include "codeGen/Quadruple.h"
#include "codeGen/RegisterAllocator.h"
#include <filesystem>
#include <iostream>
#include <cassert>

extern std::filesystem::path binaryPath;
const std::string Compiler::extension = "s";
void Compiler::externalCommand(std::filesystem::path lat) {
	auto obj = lat;
	obj.replace_extension("o");
	auto cmd = std::string("as --32 '") + lat.string() + "' -o '" + obj.string() + "'";
	cerr << cmd << endl;
	system(cmd.data());
	auto lib = binaryPath.parent_path() / "lib";
	auto runtime = lib / "runtime.o";
	auto result = obj;
	result.replace_extension("");
	cmd = std::string("ld -m elf_i386 -L'") + lib.string() + "' '" + runtime.string() + "' '" + obj.string() + "' -o '" + result.string() + "'";
	cerr << cmd << endl;
	system(cmd.data());
}

string Compiler::compile(Visitable *v) {
	buf <<
		".globl _start\n"
	    ".globl exit\n"
		"_start:\n"
		"	pushl %ebp\n"
		"	movl %esp, %ebp\n"
		"	call main\n"
		"	pushl %eax\n"
		"	call exit\n";

	for (const auto& c : scope->classes) {
		c->calculateSize();
	}

	for (const auto& f : scope->functions) {
		compileFunction(f);
	}

	return buf.str();
}

void Compiler::compileFunction(const FunctionInfoPtr& f) {
	string name = mangleFunctionName(f);
	buf << ".globl \"" << name << "\"\n";

	if (!f->block) return;

	// TODO: jeśli funkcja jest klasowa, ustawić self na ... wartosc z ecx
	// c++ thiscall ecx, stack right-to-left (push c, push b, push a)

	buf << "\"" << name << "\":\n";

	auto quadEnv = quadGen.compileFunction(f);

	RegisterAllocator regGen(quadEnv);

	for (const auto& b : quadEnv.blocks) {
		for (const auto& q : b->quads) {
			buf << q->toString() << endl;
		}
	}
}
