#ifndef ZAD2_BASICBLOCK_H
#define ZAD2_BASICBLOCK_H
#include "../TypeDefs.h"
#include <utility>
#include <vector>
#include <map>
#include <cassert>
#include "Variable.h"

using namespace std;

class BasicBlock : public std::enable_shared_from_this<BasicBlock> {
public:
	struct FlowAnalysisData {
		set<VariablePtr> in, out, def, use;
	};
	BasicBlock() = default;
	vector<QuadruplePtr> quads;
	vector<QuadruplePtr> afterInit;
	vector<BasicBlockPtr> in, out;
	vector<map<VariablePtr, VariablePtr>> phi;
	FlowAnalysisData flow;
	void append(const BasicBlockPtr& after) {
		out.push_back(after);
		after->in.push_back(shared_from_this());
		after->phi.emplace_back();
	}

	void addJumpInitQuad(const QuadruplePtr& q) {
		afterInit.push_back(q);
	}

	void addPhi(const BasicBlockPtr& blk, const VariablePtr& local, VariablePtr remote) {
		assert(blk && local && remote);
		assert(local->info && !local->info->isInstanceVariable());
		auto it = find(in.begin(), in.end(), blk);
		if (it == in.end()) throw runtime_error("blk not found in incoming blocks");
		auto mit = phi.begin() + (it - in.begin());
		(*mit)[local] = std::move(remote);
	}

	void finishQuads();
};


#endif //ZAD2_BASICBLOCK_H
